%% 4)
%
% Display coordinate system
% -------------------------
%
% We have a display coordinate frame whose origin is at the upper left
% corner of the screen. The x-axis points towards the upper right corner of
% the screen. The y-axis points downwards to the lower left corner of the
% screen.
%
%     0 ----- x
%     |
%     |
%     y
%
% When showing an image (stored as a matrix) then a row of the matrix is
% specified in terms of y-coordinates; subsequently a column is specified
% by x-coordinates.
%
%
% Image coordinate system
% ------------------------
%
% We define a coordinate system that is attached to the "parrot" part in 
% the debris image. Thus, if we can find the transformation between display
% coordinates and image coordinates, we can restore the "parrot" in the
% display coordinate system.
%
%     0 ----- u
%     |
%     |
%     v
%
%
% Mathematics
% -----------
%
% It is probably easiest to look at the coordinate systems on the screen
% as if you were in headstand looking at the back of your screen. Then the
% coordinate systems present themselves as known from paper where the
% x-axis points right and the y-axis points up.
% 
%    y              v
%    |              |
%    |              |
%    0 ----- x      0 ----- u
%
% The idea now is to let the user specify two points in the debris image
% in order to fix the three degrees of freedom (translation and rotation).
% This shall be the origin of the image coordinate system w.r.t. display
% coordinate system (op_x, op_y), and the other corner (up_x, up_y) that
% lies on the u-axis of the image coordinate system.


% Debris image
D = imread('parrots_gray_broken.png');

f = figure;
imshow(D);

R1 = restore_image(D, [300 300]);
R2 = restore_image(D, [300 300]);
R = [R1 R2];

figure;
imshow(R);