function [ R ] = restore_image( D, sz)
%RESTORE_IMAGE  Translates and rotates an image based on user input. See
%"Exercises in Basic Mathematical Tools, 
% Exercise Sheet 2, Assignment 4".

    % Restored image
    R = zeros(sz, 'uint8');

    % select the upper left corner of the parrot image
    [op_x, op_y] = ginput(1);
    % select the upper right corner of the parrot image
    [up_x, up_y] = ginput(1);

    line([op_x, up_x], [op_y, up_y], 'LineWidth', 4, 'Color', 'b');
    line([0, op_x], [0, op_y], 'LineWidth', 4, 'Color', 'r');

    theta = atan2((up_y - op_y), (up_x - op_x));

    % Define the transformation matrix in homogeneous coordinates
    H = [  
        cos(theta)    -sin(theta)   op_x
        sin(theta)     cos(theta)   op_y
             0              0         1
    ];

    for i = 1:size(R, 1)
        for j = 1:size(R, 1)
            % Transform the center of the pixel (i, j).
            % Compute (x, y) coordinates from row and column indices.
            x = j - 0.5;
            y = i - 0.5;
            % Compute the (u, v)-coordinates in the image coordinate system.
            uv = H * [x, y, 1]';
            u = uv(1);
            v = uv(2);
            % Perform bilinear interpolation if (u, v) inside debris image.
            % Note that the treatment of border pixels is not perfect and
            % might introduce some artefacts.
            if u < size(D, 2) && v < size(D, 1) && u >= 1 && v >= 1
                u1 = floor(u);
                v1 = floor(v);
                r1 = D(v1,     u1) * (u1 + 1 - u) + D(v1,     u1 + 1) * (u - u1);
                r2 = D(v1 + 1, u1) * (u1 + 1 - u) + D(v1 + 1, u1 + 1) * (u - u1);
                R(i, j) =       r1 * (v1 + 1 - v) +                r2 * (v - v1);
            end
        end
    end
end

